/*
 * Sixth 3D engine. Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */


package eu.svjatoslav.sixth.e3d.gui;

import eu.svjatoslav.sixth.e3d.geometry.GeometryCoordinate;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.geometry.TransformPipe;

public class UserRelativityTracker {

    public final static int minimalSliceFactor = 5;
    public GeometryCoordinate center = new GeometryCoordinate();
    public GeometryCoordinate right;
    public GeometryCoordinate down;

    public UserRelativityTracker() {

    }

    public void analyze(final TransformPipe transformPipe,
                        final RenderingContext renderingContext) {

        center.transform(transformPipe, renderingContext);

        if (right != null) {
            right.transform(transformPipe, renderingContext);
            down.transform(transformPipe, renderingContext);
        }
    }

    public void enableOrientationTracking() {
        right = new GeometryCoordinate(new Point3D(10, 0, 0));
        down = new GeometryCoordinate(new Point3D(0, 10, 0));
    }

    public double getAngleXY() {
        return center.transformedCoordinate
                .getAngleXY(down.transformedCoordinate);
    }

    public double getAngleXZ() {
        return center.transformedCoordinate
                .getAngleXZ(right.transformedCoordinate);
    }

    public double getAngleYZ() {
        return center.transformedCoordinate
                .getAngleYZ(down.transformedCoordinate);
    }

    public double getDistanceToUser() {
        return center.transformedCoordinate
                .getDistanceTo(Point3D.ZERO);
    }

    public double proposeSliceFactor() {
        final double distanceToCamera = getDistanceToUser();

        double proposedSliceFactor = distanceToCamera / 5;

        if (proposedSliceFactor < minimalSliceFactor)
            proposedSliceFactor = minimalSliceFactor;

        return proposedSliceFactor;
    }

}
