/*
 * Sixth 3D engine. Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui.humaninput;

import eu.svjatoslav.sixth.e3d.gui.Avatar;
import eu.svjatoslav.sixth.e3d.gui.ViewContext;
import eu.svjatoslav.sixth.e3d.gui.textEditorComponent.KeyboardHelper;

import java.awt.event.KeyEvent;

public class WorldNavigationTracker implements UserInputHandler {

    @Override
    public boolean beforeViewUpdate(final ViewContext viewContext,
                                    final int millisecondsSinceLastFrame) {

        trackKeys(millisecondsSinceLastFrame, viewContext);
        return false;
    }

    @Override
    public void focusLost(final ViewContext viewContext) {
    }

    @Override
    public void focusReceived(final ViewContext viewContext) {
    }

    @Override
    public void keyPressed(final KeyEvent event, final ViewContext viewContext) {
    }

    @Override
    public void keyReleased(final KeyEvent event, final ViewContext viewContext) {
    }

    /**
     * interpret currently pressed keys
     */
    public void trackKeys(final long millisecondsSinceLastFrame,
                          final ViewContext viewContext) {

        final UserInputTracker inputTracker = viewContext.getUserInputTracker();

        final Avatar avatar = viewContext.getAvatar();

        final double actualAcceleration = millisecondsSinceLastFrame
                * avatar.avatarAcceleration
                * (1 + (avatar.getMovementSpeed() / 10));

        if (inputTracker.isKeyPressed(KeyboardHelper.UP))
            avatar.getMovementDirection().z += actualAcceleration;

        if (inputTracker.isKeyPressed(KeyboardHelper.DOWN))
            avatar.getMovementDirection().z -= actualAcceleration;

        if (inputTracker.isKeyPressed(KeyboardHelper.RIGHT))
            avatar.getMovementDirection().x += actualAcceleration;

        if (inputTracker.isKeyPressed(KeyboardHelper.LEFT))
            avatar.getMovementDirection().x -= actualAcceleration;

        avatar.enforceSpeedLimit();
    }
}
