/*
 * Sixth 3D engine. Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.ForwardOrientedTexture;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.textcanvas.TextCanvas;
import eu.svjatoslav.sixth.e3d.renderer.raster.texture.Texture;

public class ForwardOrientedTextBlock extends ForwardOrientedTexture {

    public ForwardOrientedTextBlock(final Point3D point, final double scale,
                                    final int maxUpscaleFactor, final String text,
                                    final eu.svjatoslav.sixth.e3d.renderer.raster.Color textColor) {
        super(point, scale, getTexture(text, maxUpscaleFactor, textColor));

    }

    public static Texture getTexture(final String text,
                                     final int maxUpscaleFactor,
                                     final eu.svjatoslav.sixth.e3d.renderer.raster.Color textColor) {

        final Texture texture = new Texture(text.length()
                * TextCanvas.FONT_CHAR_WIDTH, TextCanvas.FONT_CHAR_HEIGHT,
                maxUpscaleFactor);

        // texture.graphics.setColor(Color.BLUE);
        // texture.graphics.fillRect(0, 0, texture.primaryBitmap.width,
        // texture.primaryBitmap.width);

        texture.graphics.setFont(TextCanvas.FONT);
        texture.graphics.setColor(textColor.toAwtColor());

        for (int c = 0; c < text.length(); c++)
            texture.graphics.drawChars(new char[]{text.charAt(c),}, 0, 1,
                    (c * TextCanvas.FONT_CHAR_WIDTH) - 0,
                    (0 * TextCanvas.FONT_CHAR_HEIGHT) + 11);

        return texture;
    }
}
