/*
 * Sixth 3D engine. Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.geometry.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.Color;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.GlowingPoint;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Galaxy extends AbstractCompositeShape {

    private static List<Color> colors;

    public Galaxy(final int size, final int tailCount, final int starsCount,
                  Transform transform) {

        super(transform);

        ensureColorsAreInitialized();

        final double angle1 = Math.random() * 10;
        final double angle2 = Math.random() * 10;

        final double angleSin1 = Math.sin(angle1);
        final double angleCos1 = Math.cos(angle1);
        final double angleSin2 = Math.sin(angle2);
        final double angleCos2 = Math.cos(angle2);

        Random random = new Random();

        for (int i = 1; i < starsCount; i++) {
            final double b = Math.random() * 10;

            final double s = (b * b) / 30;

            final double v1 = (Math.random() * (11.5 - b)) / 3;
            final double v1p = v1 / 2;

            final double ane = ((Math.random() * (s / 2)) / tailCount) * 2;
            final double sba = ((2 * Math.PI) / tailCount)
                    * random.nextInt(tailCount);

            final double x = (((Math.sin((b - sba) + ane) * s) + (Math.random() * v1)) - v1p)
                    * size;
            final double z = (((Math.cos((b - sba) + ane) * s) + (Math.random() * v1)) - v1p)
                    * size;
            final double y = ((Math.random() * v1) - v1p) * size;

            final double x1 = (x * angleCos1) + (z * angleSin1);
            final double z1 = (z * angleCos1) - (x * angleSin1);

            final double y1 = (y * angleCos2) + (z1 * angleSin2);
            final double z2 = (z1 * angleCos2) - (y * angleSin2);

            addStar(new Point3D(x1,y1,z2));
        }
    }

    private void addStar(final Point3D starLocation) {
        addShape(
                new GlowingPoint(starLocation,10,
                        colors.get((int) (Math.random() * colors.size()))));
    }

    private synchronized void ensureColorsAreInitialized() {
        if (colors != null) return;

        colors = new ArrayList<>();

        for (int i = 0; i < 30; i++)
            colors.add(
                    new Color(
                            Math.random() + 0.5,
                            Math.random() + 0.5,
                            Math.random() + 0.5,
                            255));
    }

}
