/*
 * Sixth 3D engine. Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.texture;

import eu.svjatoslav.sixth.e3d.renderer.raster.Color;

public class TextureBitmap {

    /**
     * Byte order: ABGR
     */
    public final byte[] bytes;

    public final int width;

    public final int height;

    public double multiplicationFactor;

    public TextureBitmap(final int width, final int height, final byte[] bytes,
                         final double multiplicationFactor) {

        this.width = width;
        this.height = height;
        this.bytes = bytes;
        this.multiplicationFactor = multiplicationFactor;
    }

    public TextureBitmap(final int width, final int height,
                         final double multiplicationFactor) {

        this(width, height, new byte[width * height * 4], multiplicationFactor);
    }

    public void drawPixel(int textureOffset,
                          final byte[] targetRenderingAreaBytes, int targetRenderingAreaOffset) {

        final int textureAlpha = bytes[textureOffset] & 0xff;

        if (textureAlpha == 0)
            return;

        if (textureAlpha == 255) {
            // skip reading of background for fully opaque pixels
            targetRenderingAreaBytes[targetRenderingAreaOffset] = (byte) 255;

            targetRenderingAreaOffset++;
            textureOffset++;
            targetRenderingAreaBytes[targetRenderingAreaOffset] = bytes[textureOffset];

            targetRenderingAreaOffset++;
            textureOffset++;
            targetRenderingAreaBytes[targetRenderingAreaOffset] = bytes[textureOffset];

            targetRenderingAreaOffset++;
            textureOffset++;
            targetRenderingAreaBytes[targetRenderingAreaOffset] = bytes[textureOffset];
            return;
        }

        final int backgroundAlpha = 255 - textureAlpha;
        textureOffset++;

        targetRenderingAreaBytes[targetRenderingAreaOffset] = (byte) 255;
        targetRenderingAreaOffset++;

        targetRenderingAreaBytes[targetRenderingAreaOffset] = (byte) ((((targetRenderingAreaBytes[targetRenderingAreaOffset] & 0xff) * backgroundAlpha) + ((bytes[textureOffset] & 0xff) * textureAlpha)) / 256);
        textureOffset++;
        targetRenderingAreaOffset++;

        targetRenderingAreaBytes[targetRenderingAreaOffset] = (byte) ((((targetRenderingAreaBytes[targetRenderingAreaOffset] & 0xff) * backgroundAlpha) + ((bytes[textureOffset] & 0xff) * textureAlpha)) / 256);
        textureOffset++;
        targetRenderingAreaOffset++;

        targetRenderingAreaBytes[targetRenderingAreaOffset] = (byte) ((((targetRenderingAreaBytes[targetRenderingAreaOffset] & 0xff) * backgroundAlpha) + ((bytes[textureOffset] & 0xff) * textureAlpha)) / 256);
    }

    public void drawPixel(final int x, final int y, final Color color) {
        int address = getAddress(x, y);

        bytes[address] = (byte) color.a;

        address++;
        bytes[address] = (byte) color.b;

        address++;
        bytes[address] = (byte) color.g;

        address++;
        bytes[address] = (byte) color.r;
    }

    public void drawRectangle(int x1, final int y1, int x2, final int y2,
                              final Color color) {

        if (x1 > x2) {
            final int tmp = x1;
            x1 = x2;
            x2 = tmp;
        }

        if (y1 > y2) {
            final int tmp = x1;
            x1 = x2;
            x2 = tmp;
        }

        for (int y = y1; y < y2; y++)
            for (int x = x1; x < x2; x++)
                drawPixel(x, y, color);
    }

    public void fillColor(final Color color) {
        int address = 0;
        while (address < bytes.length) {
            bytes[address] = (byte) color.a;
            address++;

            bytes[address] = (byte) color.b;
            address++;

            bytes[address] = (byte) color.g;
            address++;

            bytes[address] = (byte) color.r;
            address++;
        }
    }

    public int getAddress(int x, int y) {
        if (x < 0)
            x = 0;

        if (x >= width)
            x = width - 1;

        if (y < 0)
            y = 0;

        if (y >= height)
            y = height - 1;

        return ((y * width) + x) * 4;
    }
}
